from airflow.models import BaseOperator
from sqlalchemy import text

# Absolute import – this works in Airflow 3.x
from airflow_provider_iris.hooks.iris_hook import IrisHook


class IrisSQLOperator(BaseOperator):
    template_fields = ("sql",)

    def __init__(self, sql, iris_conn_id="iris_default", autocommit=True, **kwargs):
        super().__init__(**kwargs)
        self.sql = sql
        self.iris_conn_id = iris_conn_id
        self.autocommit = autocommit

    def execute(self, context):
        hook = IrisHook(iris_conn_id=self.iris_conn_id)
        engine = hook.get_engine()

        with engine.connect() as conn:
            result = conn.execute(text(self.sql))

            if self.sql.strip().upper().startswith(("SELECT", "WITH")):
                rows = result.fetchall()
                self.log.info(f"Query returned {len(rows)} rows → {rows[:10]}")

            if self.autocommit:
                conn.commit()

        engine.dispose()


# File: plugins/airflow_provider_intersystems_iris/hooks/iris_hook.py

from airflow.hooks.base import BaseHook
from typing import Any
from sqlalchemy import create_engine

class IrisHook(BaseHook):
    conn_name_attr = "iris_conn_id"
    default_conn_name = "iris_default"
    conn_type = "iris"
    hook_name = "InterSystems IRIS"

    @classmethod
    def get_ui_field_behaviour(cls) -> dict[str, Any]:
        """Return custom field behaviour."""
        return {
            "hidden_fields": [],
            "relabeling": {"login": "Username", "schema": "Namespace"},
            "placeholders": {
                "port": "1972",
            },
        }

    def __init__(self, iris_conn_id: str = "iris_default"):
        super().__init__()
        self.iris_conn_id = iris_conn_id

    def get_engine(self):
        # This will use the Airflow connection you created (iris_default)
        conn = self.get_connection(self.iris_conn_id)

        # Build the SQLAlchemy-IRIS connection URL
        url = f"iris://{conn.login}:{conn.password}@{conn.host}:{conn.port or 1972}/{conn.schema or 'USER'}"

        return create_engine(url, future=True, echo=False)


from typing import Any, Dict


def get_provider_info() -> Dict[str, Any]:
    return {
        "package-name": "airflow-provider-iris",
        "name": "IRIS Airflow Provider",
        "description": "Intersystems IRIS provider for Apache Airflow.",
        "hook-class-names": ["airflow_provider_iris.hooks.iris_hook.IrisHook"],
        "extra-links": [],
        "versions": ["0.1.0"],
    }



from setuptools import setup
from pathlib import Path
__version__ = "0.2.4"


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

"""Perform the package airflow-provider-iris setup."""
setup(
    name="airflow-provider-iris",
    version=__version__,
    description="airflow provider for intersystems",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={
        "apache_airflow_provider": [
            "provider_info=airflow_provider_iris.__init__:get_provider_info"
        ]
    },
    license="Apache License 2.0",
    packages=[
        "airflow_provider_iris",
        "airflow_provider_iris.hooks",
        "airflow_provider_iris.operators",       
    ],
    install_requires=["apache-airflow>=2.0", "sqlalchemy-iris", "pandas"],
    setup_requires=["setuptools", "wheel"],
    author="Muhammad Waseem",
    author_email="muhammadwas@outlook.com",
    url="https://github.com/mwaseem75",
    classifiers=[
        "Framework :: Apache Airflow",
        "Framework :: Apache Airflow :: Provider",
    ],
    python_requires="~=3.7",
)
